#version 430 core

layout (location = 0) out vec4 color;
layout (location = 1) out vec4 BrightColor;

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
        vec3 TangentLightPos1;
	vec3 TangentLightPos2;
	vec3 TangentLightPos3;
	vec3 TangentLightPos4;
        vec3 TangentViewPos;
        vec3 TangentFragPos;
} vert;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform sampler2D normalMap;
uniform sampler2D lightMap;
uniform bool norm;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;

uniform struct PointLight2 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL2;

uniform struct PointLight3 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL3;

vec3 phong(vec3 texColor,vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	//return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
	return (specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	

	vec3 texColor=texture(diffuseTexture, vert.uv).rgb;
	vec3 lightMapColor =texture(lightMap, vert.uv).rgb;
	texColor.x*=lightMapColor.x*lightMapColor.x*lightMapColor.x;
	texColor.y*=lightMapColor.y*lightMapColor.y*lightMapColor.y;
	texColor.z*=lightMapColor.z*lightMapColor.z*lightMapColor.z;
     
if(norm== true){  

   // obtain normal from normal map in range [0,1]
    	vec3 normal = texture(normalMap, vert.uv).rgb;
   // transform normal vector to range [-1,1]
    	normal = normalize(normal * 2.0 - 1.0);  // this normal is in tangent space
   
   // ambient
    	color = vec4 (materialCoefficients.x * texColor,1);

   // diffuse1
    	vec3 lightDir1 = normalize(vert.TangentLightPos1 - vert.TangentFragPos);
	
	
    	float diff1 = max(dot(lightDir1, normal), 0.0);
    	diff1 = diff1*materialCoefficients.y;
   	vec3 diffuse1 = texColor;

   // diffuse2
    	vec3 lightDir2 = normalize(vert.TangentLightPos2 - vert.TangentFragPos);
	
	
    	float diff2 = max(dot(lightDir2, normal), 0.0);
	diff2 = diff2*materialCoefficients.y;
   	 vec3 diffuse2 = texColor;

   // diffuse3
    	vec3 lightDir3 = normalize(vert.TangentLightPos3 - vert.TangentFragPos);
	
	
    	float diff3 = max(dot(lightDir3, normal), 0.0);
	diff3 = diff3*materialCoefficients.y;
    	vec3 diffuse3 = texColor;

   // diffuse4
    	vec3 lightDir4 = normalize(vert.TangentLightPos4 - vert.TangentFragPos);
	
	
    	float diff4 = max(dot(lightDir4, normal), 0.0);
	diff4 = diff4*materialCoefficients.y;
    	vec3 diffuse4= texColor;

    // specular1
    	vec3 v = normalize(vert.TangentViewPos - vert.TangentFragPos);
    	vec3 r = reflect(-lightDir1, normal);
	float spec1= pow(max(0,dot(v,r)),specularAlpha); 
	vec3 specular1= materialCoefficients.z*vec3(1.0,1.0,1.0)*spec1;

    // specular2
    	vec3 r2 = reflect(-lightDir2, normal);
	float spec2= pow(max(0,dot(v,r2)),specularAlpha); 
	vec3 specular2= materialCoefficients.z*vec3(1.0,1.0,1.0)*spec2;

    // specular3  	
    	vec3 r3 = reflect(-lightDir3, normal);
	float spec3= pow(max(0,dot(v,r3)),specularAlpha); 
	vec3 specular3= materialCoefficients.z*vec3(1.0,1.0,1.0)*spec3;

    // specular4
    	vec3 r4 = reflect(-lightDir4, normal);
	float spec4= pow(max(0,dot(v,r4)),specularAlpha); 
	vec3 specular4= materialCoefficients.z*vec3(1.0,1.0,1.0)*spec4;

 	float d1= length(pointL.position - vert.position_world);
 	float att1 = 1.0f / (pointL.attenuation.x + d1 * pointL.attenuation.y + d1 * d1 * pointL.attenuation.z);
 	float d2= length(pointL2.position - vert.position_world);
 	float att2 = 1.0f / (pointL2.attenuation.x + d2 * pointL2.attenuation.y + d2 * d2 * pointL2.attenuation.z);
 	float d3= length(pointL3.position - vert.position_world);
 	float att3 = 1.0f / (pointL3.attenuation.x + d3 * pointL3.attenuation.y + d3 * d3 * pointL3.attenuation.z);

   	color.rgb += (diffuse1 + specular1)*att1*0.6*pointL.color.x;
    	color.rgb+=(diffuse2 + specular2)*att2*0.6*pointL2.color.x;
   	color.rgb+=(diffuse3 + specular3)*att3*0.6*pointL3.color.x;
    	color.rgb+=(diffuse4 + specular4)*0.6*dirL.color.x;

}else{


	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	//vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	color = vec4(texColor * materialCoefficients.x, 1); // ambient
	color.rgb+=texColor*materialCoefficients.y; //diffuse
	//vec3 phong(vec3 texColor,vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {

	// add directional light contribution
	color.rgb += phong(texColor,n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	// add point light contribution
	color.rgb += phong(texColor,n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);
	
	// add point light 2 contribution
	color.rgb += phong(texColor,n, pointL2.position - vert.position_world, v, pointL2.color * texColor, materialCoefficients.y, pointL2.color, materialCoefficients.z, specularAlpha, true, pointL2.attenuation);
	
	// add point light 3 contribution
	color.rgb += phong(texColor,n, pointL3.position - vert.position_world, v, pointL3.color * texColor, materialCoefficients.y, pointL3.color, materialCoefficients.z, specularAlpha, true, pointL3.attenuation);
 

}



	// check whether result is higher than some threshold, if so, output as bloom threshold color    	
	float brightness = dot(color.rgb, vec3(0.2126, 0.7152, 0.0722));
    	if(brightness > 1.0)
        	BrightColor = color;
    	else
       		BrightColor = vec4(0.0, 0.0, 0.0, 1.0);
	
}