#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout (location = 3) in vec3 tangent;
layout (location = 4) in vec3 bitangent;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
        vec3 TangentLightPos1;
	vec3 TangentLightPos2;
	vec3 TangentLightPos3;
	vec3 TangentLightPos4;
        vec3 TangentViewPos;
        vec3 TangentFragPos;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;

uniform vec3 lightPos1;
uniform vec3 lightPos2;
uniform vec3 lightPos3;
uniform vec3 lightDirection;
uniform vec3 viewPos;

void main() {
	vert.normal_world = normalMatrix * normal;
	vec2 uv2 = uv;
	uv2.y=-uv2.y;
	vert.uv = uv2;
	vec4 position_world_ = modelMatrix * vec4(position, 1);
	vert.position_world = position_world_.xyz;


	vec3 directionalLightPos = position+lightDirection;

	vec3 T = normalize(normalMatrix * tangent);
        vec3 N = normalize(normalMatrix * normal);
        T = normalize(T - dot(T, N) * N);
        vec3 B = cross(N, T);

	mat3 TBN = transpose(mat3(T, B, N));    
        vert.TangentLightPos1 = TBN * lightPos1;
	vert.TangentLightPos2 = TBN * lightPos2;
	vert.TangentLightPos3 = TBN * lightPos3;
	vert.TangentLightPos4 = TBN * directionalLightPos;
        vert.TangentViewPos  = TBN * viewPos;
        vert.TangentFragPos  = TBN * vert.position_world;


	gl_Position = viewProjMatrix * position_world_;
}