#version 430 core

layout (location = 0) out vec4 color;
layout (location = 1) out vec4 BrightColor;

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;


uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
//uniform sampler2D diffuseTexture;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;

uniform struct PointLight2 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL2;

uniform struct PointLight3 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL3;

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}


	vec3 perm (vec3 x) { return mod(((x*33.0)+1.0)*x, 288.0); }

	float genTex(vec2 v){
  	const vec4 color = vec4(0.211324865405187, 0.366025403784439,
          	 -0.577350269189626, 0.024390243902439);
	
  	vec2 i  = floor(v + dot(v, color.yy) );
	vec2 j = v -   i + dot(i, color.xx); 
 	vec2 k; 

  	k = (j.x > j.y) ? vec2(1.0, 0.0) : vec2(0.0, 1.0);
  	vec4 l = j.xyxy + color.xxzz; 
  	l.xy -= k;
  		i = mod(i, 289.0);
 	vec3 p = perm( perm( i.y + vec3(0.0, k.y, 1.0 ))
 		 + i.x + vec3(0.0, k.x, 1.0 ));
  	vec3 m = max(0.5 - vec3(dot(j,j), dot(l.xy,l.xy),
    		dot(l.zw,l.zw)), 0.0);
  		m = m*m ;
  		m = m*m ;

  	vec3 n = 2.0 * fract(p * color.www) - 1.0; 
  	vec3 ab = abs(n) - 0.5; 
  	vec3 flo = floor(n + 0.5); 
  	vec3 a = n - flo; 
  		
		m *= 1.58495342379584 - 0.58375960582456 * ( a*a + ab*ab );
  	vec3 g;
 		g.x  = a.x  * j.x  + ab.x  * j.y;
  		g.yz = a.yz * l.xz + ab.yz * l.yw;
 	return 125.0 * dot(m, g);
}


void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	
	vec3 texColor;
	
	vec2 uv = vec2( vert.uv.x*40,vert.uv.y*40*40);
       	float value = genTex (uv);
	
	
	texColor = vec3(0.85+value*0.15,0.75+value*0.25,0.55+value*0.45);

	color = vec4(texColor * materialCoefficients.x, 1); // ambientvec4 
	
	// add directional light contribution
	color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	// add point light contribution
	color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);
	
	// add point light 2 contribution
	color.rgb += phong(n, pointL2.position - vert.position_world, v, pointL2.color * texColor, materialCoefficients.y, pointL2.color, materialCoefficients.z, specularAlpha, true, pointL2.attenuation);
	
	// add point light 3 contribution
	color.rgb += phong(n, pointL3.position - vert.position_world, v, pointL3.color * texColor, materialCoefficients.y, pointL3.color, materialCoefficients.z, specularAlpha, true, pointL3.attenuation);
	

	// check whether result is higher than some threshold, if so, output as bloom threshold color    	
	float brightness = dot(color.rgb, vec3(0.2126, 0.7152, 0.0722));
    	if(brightness > 1.0)
        	BrightColor = color;
    	else
       		BrightColor = vec4(0.0, 0.0, 0.0, 1.0);

}

