#version 430 core

layout (location = 0) out vec4 color;
layout (location = 1) out vec4 BrightColor;

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	vec3 TangentLightPos;
        vec3 TangentViewPos;
        vec3 TangentFragPos;
} vert;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform sampler2D lightMap;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;

uniform struct PointLight2 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL2;

uniform struct PointLight3 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL3;

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return ( specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);

	vec3 lightMap = texture(lightMap, vert.uv).rgb;
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	texColor.x*=lightMap.x*lightMap.x;
	texColor.y*=lightMap.y*lightMap.y;
	texColor.z*=lightMap.z*lightMap.z;

	color = vec4(texColor * materialCoefficients.x, 1); // ambient
	color.rgb+=texColor*materialCoefficients.y; //diffuse
	// add directional light contribution
	color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	// add point light contribution
	color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);
	
	// add point light 2 contribution
	color.rgb += phong(n, pointL2.position - vert.position_world, v, pointL2.color * texColor, materialCoefficients.y, pointL2.color, materialCoefficients.z, specularAlpha, true, pointL2.attenuation);
	
	// add point light 3 contribution
	color.rgb += phong(n, pointL3.position - vert.position_world, v, pointL3.color * texColor, materialCoefficients.y, pointL3.color, materialCoefficients.z, specularAlpha, true, pointL3.attenuation);
 
	//bloom	
	vec3 color2= vec3(color[0],color[1],color[2]);
	float brightness = dot(color2, vec3(0.2126, 0.7152, 0.0722));
    	if(brightness > 1.0)
        	BrightColor = color;
    	else
       		BrightColor = vec4(0.0, 0.0, 0.0, 1.0);
	
}

